@@ -34,6 +34,15 @@ struct Track {
   } contents;
 };
 
+typedef enum {
+	CDRECORDER_EJECT			= 1 << 0,
+	CDRECORDER_BLANK			= 1 << 1,
+	CDRECORDER_DUMMY_WRITE			= 1 << 2,
+	CDRECORDER_DISC_AT_ONCE			= 1 << 3,
+	CDRECORDER_DEBUG			= 1 << 4,
+	CDRECORDER_OVERBURN			= 1 << 5
+} CDRecorderWriteFlags;
+
 #define CD_TYPE_RECORDER            (cd_recorder_get_type ())
 #define CD_RECORDER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), CD_TYPE_RECORDER, CDRecorder))
 #define CD_RECORDER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), CD_TYPE_RECORDER, CDRecorderClass))
@@ -59,9 +68,9 @@ struct CDRecorderClass {
 	void (*animation_changed)	(CDRecorder *cdrecorder,
 					 gboolean spinning);
 	void (*insert_cd_request)	(CDRecorder *cdrecorder,
-					 gboolean is_reload);
-	void (*insert_cd_rw_request)	(CDRecorder *cdrecorder,
-					 gboolean is_reload);
+					 gboolean is_reload,
+					 gboolean can_rewrite,
+					 gboolean busy_cd);
 };
 
 GType		cd_recorder_get_type		(void);
@@ -69,14 +78,11 @@ CDRecorder     *cd_recorder_new			(void);
 
 GList	       *cd_recorder_list_actions	(CDRecorder *cdrecorder);
 
-int		cd_recorder_write_tracks		(CDRecorder *cdrecorder,
-						 CDDrive *recorder,
+int		cd_recorder_write_tracks	(CDRecorder *cdrecorder,
+						 CDDrive *drive,
 						 GList *tracks,
 						 gint speed,
-						 gboolean eject,
-						 gboolean blank,
-						 gboolean dummy_write,
-						 gboolean disc_at_once);
+						 CDRecorderWriteFlags flags);
 
 gboolean	cd_recorder_cancel		(CDRecorder *cdrecorder,
 						 gboolean skip_if_dangerous);
@@ -84,8 +90,6 @@ gboolean	cd_recorder_cancel		(CDRecorder *cdrecorder,
 const char*	cd_recorder_get_error_message	(CDRecorder *cdrecorder);
 const char*	cd_recorder_get_error_message_details
 						(CDRecorder *cdrecorder);
-void		cd_recorder_insert_cd_retry	(CDRecorder *cdrecorder,
-						 gboolean is_reload);
 void		cd_recorder_track_free		(Track *track);
 
 #endif /* CD_RECORDER_H */
