@@ -28,7 +28,6 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-ui-init.h>
-#include <eel/eel-vfs-extensions.h>
 #include <gconf/gconf-client.h>
 #include <time.h>
 #include <stdio.h>
@@ -57,6 +56,46 @@ static int cancel = CANCEL_NONE;
 /* For the image spinning */
 static GList *image_list;
 
+/* Originally eel_make_valid_utf8 */
+static char *
+ncb_make_valid_utf8 (const char *name)
+{
+	GString *string;
+	const char *remainder, *invalid;
+	int remaining_bytes, valid_bytes;
+
+	string = NULL;
+	remainder = name;
+	remaining_bytes = strlen (name);
+
+	while (remaining_bytes != 0) {
+		if (g_utf8_validate (remainder, remaining_bytes, &invalid)) {
+			break;
+		}
+		valid_bytes = invalid - remainder;
+
+		if (string == NULL) {
+			string = g_string_sized_new (remaining_bytes);
+		}
+		g_string_append_len (string, remainder, valid_bytes);
+		g_string_append_c (string, '?');
+
+		remaining_bytes -= valid_bytes + 1;
+		remainder = invalid + 1;
+	}
+
+	if (string == NULL) {
+		return g_strdup (name);
+	}
+
+	g_string_append (string, remainder);
+	g_string_append (string, _(" (invalid Unicode)"));
+	g_assert (g_utf8_validate (string->str, -1, NULL));
+
+	return g_string_free (string, FALSE);
+}
+
+
 GtkWindow *
 cd_progress_get_window (void)
 {
@@ -77,10 +116,12 @@ void
 cd_progress_set_text (const char *text)
 {
 	GtkWidget *progress_label;
-	
-	progress_label = glade_xml_get_widget (xml, "cd_progress_label");
+	char *string;
 
-	gtk_label_set_text (GTK_LABEL (progress_label), text);
+	progress_label = glade_xml_get_widget (xml, "cd_progress_label");
+	string = g_strdup_printf ("<i>%s</i>", text);
+	gtk_label_set_markup (GTK_LABEL (progress_label), string);
+	g_free (string);
 }
 
 static gboolean
@@ -198,190 +239,50 @@ progress_changed_cb (CDRecorder *cdrecorder, gdouble fraction, gpointer data)
 	cd_progress_set_fraction (fraction);
 }
 
-static void
-reload_dialog_response (GtkDialog *dialog, gint response_id, gpointer data)
-{
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-
-	cd_recorder_insert_cd_retry (cdrecorder, TRUE);
-}
-
-static void
-insert_cd_response (GtkDialog *dialog, gint response_id, gpointer data)
-{
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-
-	cd_recorder_insert_cd_retry (cdrecorder, FALSE);
-}
-
-static void
-insert_cdrw_response (GtkDialog *dialog, gint response_id, gpointer data)
-{
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-
-	cd_recorder_insert_cd_retry (cdrecorder, FALSE);
-}
-
-static void
-show_reload_dialog (CDRecorder *cdrecorder)
-{
-	GtkWidget *reload_dialog;
-
-	reload_dialog = gtk_message_dialog_new (cd_progress_get_window (),
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_INFO,
-						GTK_BUTTONS_OK,
-						_("Please reload the CD in the CD writer."));
-	gtk_window_set_title (GTK_WINDOW (reload_dialog),
-			      _("Reload CD"));
-	g_signal_connect (reload_dialog, "response", (GCallback)reload_dialog_response, cdrecorder);
-	gtk_widget_show (reload_dialog);
-}
-
-static void
-show_insert_dialog (CDRecorder *cdrecorder)
-{
-	GtkWidget *insert_cd_dialog;
-
-	insert_cd_dialog = gtk_message_dialog_new (cd_progress_get_window (),
-						   GTK_DIALOG_DESTROY_WITH_PARENT,
-						   GTK_MESSAGE_INFO,
-						   GTK_BUTTONS_OK,
-						   _("Please insert a blank CD in the CD writer."));
-	gtk_window_set_title (GTK_WINDOW (insert_cd_dialog),
-			      _("Insert blank CD"));
-	g_signal_connect (insert_cd_dialog, "response", (GCallback)insert_cd_response, cdrecorder);
-	gtk_widget_show (insert_cd_dialog);
-}
-
-static void
-show_insert_rw_dialog (CDRecorder *cdrecorder)
-{
-	GtkWidget *insert_cdrw_dialog;
-
-	insert_cdrw_dialog = gtk_message_dialog_new (cd_progress_get_window (),
-						   GTK_DIALOG_DESTROY_WITH_PARENT,
-						   GTK_MESSAGE_INFO,
-						   GTK_BUTTONS_OK,
-						   _("Please insert a blank CD-RW in the CD writer."));
-	gtk_window_set_title (GTK_WINDOW (insert_cdrw_dialog),
-			      _("Insert blank CD-RW"));
-	g_signal_connect (insert_cdrw_dialog, "response", (GCallback)insert_cdrw_response, cdrecorder);
-	gtk_widget_show (insert_cdrw_dialog);
-}
-
-static void
-insert_cd_request_cb (CDRecorder *cdrecorder, gboolean is_reload,
-		      gpointer data)
-{
-	if (is_reload) {
-		show_reload_dialog (cdrecorder);
-	} else {
-		show_insert_dialog (cdrecorder);
-	}
-}
-
-static void
-insert_rwcd_request_cb (CDRecorder *cdrecorder, gboolean is_reload,
-		      gpointer data)
-{
-	if (is_reload) {
-		show_reload_dialog (cdrecorder);
-	} else {
-		show_insert_rw_dialog (cdrecorder);
-	}
-}
-
 static gboolean
-media_type_matches (CDMediaType type, gboolean blank_cd)
+insert_cd_request_cb (CDRecorder *cdrecorder,
+		      gboolean is_reload, gboolean can_rewrite,
+		      gboolean busy_cd, gpointer data)
 {
-	switch (type)
-	{
-	case CD_MEDIA_TYPE_ERROR:
-	case CD_MEDIA_TYPE_BUSY:
-		return FALSE;
-	case CD_MEDIA_TYPE_UNKNOWN:
-		return TRUE;
-	case CD_MEDIA_TYPE_CD:
-	case CD_MEDIA_TYPE_DVD:
-	case CD_MEDIA_TYPE_DVD_RAM:
-		return FALSE;
-	case CD_MEDIA_TYPE_CDR:
-		return (blank_cd ? FALSE : TRUE);
-	case CD_MEDIA_TYPE_CDRW:
-		return TRUE;
-	case CD_MEDIA_TYPE_DVDR:
-		return (blank_cd ? FALSE : TRUE);
-	case CD_MEDIA_TYPE_DVDRW:
-		return TRUE;
-	case CD_MEDIA_TYPE_DVD_PLUS_R:
-		return (blank_cd ? FALSE : TRUE);
-	case CD_MEDIA_TYPE_DVD_PLUS_RW:
-		return TRUE;
+	GtkWidget *dialog, *parent;
+	const char *msg, *title;
+	int res;
+
+	if (busy_cd) {
+		msg = N_("Please make sure another application is not using the disc.");
+		title = N_("Disc is busy");
+	} else if (is_reload && can_rewrite) {
+		msg = N_("Please insert a rewritable or blank media in the drive tray.");
+		title = N_("Insert rewritable or blank media");
+	} else if (is_reload && !can_rewrite) {
+		msg = N_("Please insert a blank media in the drive tray.");
+		title = N_("Insert blank media");
+	} else if (can_rewrite) {
+			msg = N_("Please replace the in-drive media by a rewritable or blank media.");
+			title = N_("Reload rewritable or blank media");
+	} else {
+		msg = N_("Please replace the in-drive media by a blank media.");
+		title = N_("Reload blank media");
 	}
 
-	return FALSE;
-}
-
-static gboolean
-wait_for_insertion (const CDDrive *rec, gboolean blank_cd)
-{
-	CDMediaType type;
-	gboolean reload;
-
-	reload = FALSE;
-	type = cd_drive_get_media_type ((CDDrive *)rec);
-
-	if (type == CD_MEDIA_TYPE_ERROR) {
-		reload = TRUE;
-	}
-
-	while (!media_type_matches (type, blank_cd)) {
-		GtkWidget *dialog, *parent;
-		const char *msg, *title;
-		int res;
-
-		if (reload && blank_cd) {
-			msg = N_("Please insert a rewritable media in the drive tray.");
-			title = N_("Insert rewritable media");
-		} else if (reload && !blank_cd) {
-			msg = N_("Please insert a blank media in the drive tray.");
-			title = N_("Insert blank media");
-		} else if (blank_cd) {
-			msg = N_("Please replace the in-drive media by a rewritable media.");
-			title = N_("Reload rewritable media");
-		} else {
-			msg = N_("Please replace the in-drive media by a blank media.");
-			title = N_("Reload blank media");
-		}
-
-		parent = glade_xml_get_widget (xml, "cdr_dialog");
-		dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
-				GTK_DIALOG_MODAL,
-				GTK_MESSAGE_INFO,
-				GTK_BUTTONS_OK_CANCEL,
-				"%s", _(msg));
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-				GTK_RESPONSE_OK);
-		gtk_window_set_title (GTK_WINDOW (dialog), _(title));
-		res = gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-		if (res == GTK_RESPONSE_CANCEL) {
-			return FALSE;
-		}
-
-		type = cd_drive_get_media_type ((CDDrive *)rec);
-		reload = FALSE;
-		if (type == CD_MEDIA_TYPE_UNKNOWN
-				|| type == CD_MEDIA_TYPE_ERROR) {
-			reload = TRUE;
-		}
+	parent = glade_xml_get_widget (xml, "cdr_dialog");
+	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+			GTK_DIALOG_MODAL,
+			GTK_MESSAGE_INFO,
+			GTK_BUTTONS_OK_CANCEL,
+			"%s", _(msg));
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+			GTK_RESPONSE_OK);
+	gtk_window_set_title (GTK_WINDOW (dialog), _(title));
+	res = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	if (res == GTK_RESPONSE_CANCEL) {
+		return FALSE;
 	}
 
 	return TRUE;
 }
 
-
 static void
 details_clicked (GtkButton *button, gpointer data)
 {
@@ -592,7 +493,7 @@ select_iso_filename (void)
 }
 
 static int
-burn_cd (const CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, int speed, gboolean dummy, gboolean debug)
+burn_cd (const CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, int speed, gboolean dummy, gboolean debug, gboolean overburn)
 {
 	CDRecorderWriteFlags flags;
 	GList *tracks;
@@ -611,7 +512,6 @@ burn_cd (const CDDrive *rec, const char *source_iso, const char *label, gboolean
 		if (filename == NULL) {
 			return RESULT_CANCEL;
 		} else {
-
 			/* Check if you have permission to create or
 			 * overwrite the file.
 			 */
@@ -695,10 +595,6 @@ burn_cd (const CDDrive *rec, const char *source_iso, const char *label, gboolean
 	} else {
 		Track *track;
 
-		if (!wait_for_insertion (rec, blank_cd)) {
-			return RESULT_CANCEL;
-		}
-
 		cdrecorder = cd_recorder_new ();
 		cancel = CANCEL_CD_RECORD;
 
@@ -718,10 +614,6 @@ burn_cd (const CDDrive *rec, const char *source_iso, const char *label, gboolean
 				  "insert-cd-request",
 				  G_CALLBACK (insert_cd_request_cb),
 				  NULL);
-		g_signal_connect (G_OBJECT (cdrecorder),
-				  "insert-rwcd-request",
-				  G_CALLBACK (insert_rwcd_request_cb),
-				  NULL);
 
 		track = g_new0 (Track, 1);
 		track->type = TRACK_TYPE_DATA;
@@ -732,15 +624,15 @@ burn_cd (const CDDrive *rec, const char *source_iso, const char *label, gboolean
 		if (eject_cd) {
 			flags |= CDRECORDER_EJECT;
 		}
-		if (blank_cd) {
-			flags |= CDRECORDER_BLANK;
-		}
 		if (dummy) {
 			flags |= CDRECORDER_DUMMY_WRITE;
 		}
 		if (debug) {
 			flags |= CDRECORDER_DEBUG;
 		}
+		if (overburn) {
+			flags |= CDRECORDER_OVERBURN;
+		}
 
 		res = cd_recorder_write_tracks (cdrecorder, (CDDrive *)rec,
 				tracks, speed, flags);
@@ -789,7 +681,6 @@ refresh_dialog (GladeXML *xml)
 	GtkWidget *menu;
 	GtkWidget *item;
 	GtkWidget *eject_check;
-	GtkWidget *blank_check;
 	GList *l, *children;
 	GConfClient *gc;
 	char *name;
@@ -835,14 +726,10 @@ refresh_dialog (GladeXML *xml)
 	/* Disable eject for burn-to-iso */
 	eject_check = glade_xml_get_widget (xml, "eject_check");
 	gtk_widget_set_sensitive (eject_check, rec->type != CDDRIVE_TYPE_FILE);
-
-	/* Disable blank for burn-to-iso */
-	blank_check = glade_xml_get_widget (xml, "blank_check");
-	gtk_widget_set_sensitive (blank_check, rec->type & CDDRIVE_TYPE_CDRW_RECORDER);
 }
 
 static void
-target_changed (BaconCdSelection *bcs, GladeXML *xml)
+target_changed (BaconCdSelection *bcs, const char *device_path, GladeXML *xml)
 {
   refresh_dialog (xml);
 }
@@ -892,11 +779,10 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	char buf[129];
 	GDate *now;
 
-	label = glade_xml_get_widget (xml, "top_label");
+	label = glade_xml_get_widget (xml, "size_label");
 
 	size_mb = (size + 1024 * 1024 - 1)/ (1024 * 1024);
-	str = g_strdup_printf (_("You have selected %d megabyte of files to be written to the CD."),
-				size_mb);
+	str = g_strdup_printf (_("%d MB"), size_mb);
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
 
@@ -1074,7 +960,7 @@ verify_iso (const char *filename, char **iso_label)
 		/* Hmm, not UTF-8. Try the current locale. */
 		str2 = g_locale_to_utf8 (str, -1, NULL, NULL, NULL);
 		if (str2 == NULL) {
-			str2 = eel_make_valid_utf8 (str);
+			str2 = ncb_make_valid_utf8 (str);
 		}
 		g_free (str);
 		str = str2;
@@ -1101,11 +987,11 @@ int
 main (int argc, char *argv[])
 {
 	const CDDrive *rec;
-	GtkWidget *dialog, *entry, *eject_check, *blank_check, *reuse_check, *progress_window;
+	GtkWidget *dialog, *entry, *eject_check, *reuse_check, *progress_window;
 	GtkWidget *cancel_button, *dummy_check, *option_menu, *item;
 	const char *label;
 	int res, speed, i;
-	gboolean eject, blank, reuse, dummy, debug;
+	gboolean eject, reuse, dummy, debug, overburn;
 	GnomeVFSURI *uri;
 	GnomeVFSFileSize size;
 	char *source_iso, *disc_name;
@@ -1188,23 +1074,7 @@ main (int argc, char *argv[])
 			return 1;
 		}
 	}
-#if 0
-	recorders = scan_for_cdroms (TRUE, source_iso != NULL ? FALSE : TRUE);
 
-	if (recorders == NULL) {
-		dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("You do not seem to have any CD recorder in your machine. Please verify your setup."));
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
-		gtk_window_set_title (GTK_WINDOW (dialog),
-				      _("No CD recorder available"));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-		return 1;
-	}
-#endif
 	xml = glade_xml_new (DATADIR "/cdburn.glade", NULL, NULL);
 	dialog = glade_xml_get_widget (xml, "cdr_dialog");
 
@@ -1220,9 +1090,10 @@ main (int argc, char *argv[])
 
 	gc = gconf_client_get_default ();
 	debug = gconf_client_get_bool (gc, "/apps/nautilus-cd-burner/debug", NULL);
+	overburn = gconf_client_get_bool (gc, "/apps/nautilus-cd-burner/overburn", NULL);
 	g_object_unref (G_OBJECT (gc));
 
-	if (debug) {
+	if (debug == FALSE) {
 		gtk_widget_hide (dummy_check);
 	}
 
@@ -1256,9 +1127,6 @@ main (int argc, char *argv[])
 		eject_check = glade_xml_get_widget (xml, "eject_check");
 		eject = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (eject_check));
 
-		blank_check = glade_xml_get_widget (xml, "blank_check");
-		blank = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (blank_check));
-
 		dummy = FALSE;
 
 		if (debug) {
@@ -1273,7 +1141,7 @@ main (int argc, char *argv[])
 		gconf_client_set_int (gc, "/apps/nautilus-cd-burner/default_speed", speed, NULL);
 		g_object_unref (G_OBJECT (gc));
 
-		res = burn_cd (rec, source_iso, label, eject, blank, speed, dummy, debug);
+		res = burn_cd (rec, source_iso, label, eject, speed, dummy, debug, overburn);
 
 		reuse_check = glade_xml_get_widget (xml, "reuse_check");
 		reuse = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (reuse_check));
